@@ -5,13 +5,14 @@ from typing import Any
 
 from cleo.helpers import argument
 from cleo.helpers import option
+from poetry.core.version.exceptions import InvalidVersion
 from tomlkit.toml_document import TOMLDocument
 
 from poetry.console.commands.command import Command
 
 
 if TYPE_CHECKING:
-    from poetry.core.semver.version import Version
+    from poetry.core.constraints.version import Version
 
 
 class VersionCommand(Command):
@@ -26,7 +27,7 @@ class VersionCommand(Command):
             "version",
             "The version number or the rule to update the version.",
             optional=True,
-        )
+        ),
     ]
     options = [
         option("short", "s", "Output the version number only"),
@@ -35,6 +36,7 @@ class VersionCommand(Command):
             None,
             "Do not update pyproject.toml file",
         ),
+        option("next-phase", None, "Increment the phase of the current version"),
     ]
 
     help = """\
@@ -61,7 +63,7 @@ patch, minor, major, prepatch, preminor, premajor, prerelease.
 
         if version:
             version = self.increment_version(
-                self.poetry.package.pretty_version, version
+                self.poetry.package.pretty_version, version, self.option("next-phase")
             )
 
             if self.option("short"):
@@ -90,12 +92,14 @@ patch, minor, major, prepatch, preminor, premajor, prerelease.
 
         return 0
 
-    def increment_version(self, version: str, rule: str) -> Version:
-        from poetry.core.semver.version import Version
+    def increment_version(
+        self, version: str, rule: str, next_phase: bool = False
+    ) -> Version:
+        from poetry.core.constraints.version import Version
 
         try:
             parsed = Version.parse(version)
-        except ValueError:
+        except InvalidVersion:
             raise ValueError("The project's version doesn't seem to follow semver")
 
         if rule in {"major", "premajor"}:
@@ -114,7 +118,8 @@ patch, minor, major, prepatch, preminor, premajor, prerelease.
             if parsed.is_unstable():
                 pre = parsed.pre
                 assert pre is not None
-                new = Version(parsed.epoch, parsed.release, pre.next())
+                pre = pre.next_phase() if next_phase else pre.next()
+                new = Version(parsed.epoch, parsed.release, pre)
             else:
                 new = parsed.next_patch().first_prerelease()
         else:
