@@ -1,24 +1,35 @@
 from __future__ import annotations
 
+import hashlib
+import io
 import os
 import shutil
 import stat
 import sys
+import tarfile
 import tempfile
+import zipfile
 
+from collections.abc import Mapping
 from contextlib import contextmanager
+from contextlib import suppress
+from functools import cached_property
 from pathlib import Path
 from typing import TYPE_CHECKING
 from typing import Any
-from typing import Iterator
-from typing import Mapping
-from typing import cast
+from typing import overload
+
+import requests
+
+from requests.utils import atomic_open
 
 from poetry.utils.constants import REQUESTS_TIMEOUT
 
 
 if TYPE_CHECKING:
     from collections.abc import Callable
+    from collections.abc import Iterator
+    from types import TracebackType
 
     from poetry.core.packages.package import Package
     from requests import Session
@@ -36,7 +47,23 @@ def directory(path: Path) -> Iterator[Path]:
         os.chdir(cwd)
 
 
-def _on_rm_error(func: Callable[[str], None], path: str, exc_info: Exception) -> None:
+# Correct type signature when used as `shutil.rmtree(..., onexc=_on_rm_error)`.
+@overload
+def _on_rm_error(
+    func: Callable[[str], None], path: str, exc_info: Exception
+) -> None: ...
+
+
+# Correct type signature when used as `shutil.rmtree(..., onerror=_on_rm_error)`.
+@overload
+def _on_rm_error(
+    func: Callable[[str], None],
+    path: str,
+    exc_info: tuple[type[BaseException], BaseException, TracebackType],
+) -> None: ...
+
+
+def _on_rm_error(func: Callable[[str], None], path: str, exc_info: Any) -> None:
     if not os.path.exists(path):
         return
 
@@ -44,9 +71,7 @@ def _on_rm_error(func: Callable[[str], None], path: str, exc_info: Exception) ->
     func(path)
 
 
-def remove_directory(
-    path: Path | str, *args: Any, force: bool = False, **kwargs: Any
-) -> None:
+def remove_directory(path: Path, force: bool = False) -> None:
     """
     Helper function handle safe removal, and optionally forces stubborn file removal.
     This is particularly useful when dist files are read-only or git writes read-only
@@ -54,15 +79,19 @@ def remove_directory(
 
     Internally, all arguments are passed to `shutil.rmtree`.
     """
-    if Path(path).is_symlink():
-        return os.unlink(str(path))
+    if path.is_symlink():
+        return os.unlink(path)
 
-    kwargs["onerror"] = kwargs.pop("onerror", _on_rm_error if force else None)
-    shutil.rmtree(path, *args, **kwargs)
+    kwargs: dict[str, Any] = {}
+    if force:
+        onexc = "onexc" if sys.version_info >= (3, 12) else "onerror"
+        kwargs[onexc] = _on_rm_error
+
+    shutil.rmtree(path, **kwargs)
 
 
 def merge_dicts(d1: dict[str, Any], d2: dict[str, Any]) -> None:
-    for k in d2.keys():
+    for k in d2:
         if k in d1 and isinstance(d1[k], dict) and isinstance(d2[k], Mapping):
             merge_dicts(d1[k], d2[k])
         else:
@@ -75,25 +104,16 @@ def download_file(
     session: Authenticator | Session | None = None,
     chunk_size: int = 1024,
 ) -> None:
-    import requests
-
     from poetry.puzzle.provider import Indicator
 
-    get = requests.get if not session else session.get
-
-    response = get(url, stream=True, timeout=REQUESTS_TIMEOUT)
-    response.raise_for_status()
+    downloader = Downloader(url, dest, session)
 
     set_indicator = False
     with Indicator.context() as update_context:
         update_context(f"Downloading {url}")
 
-        if "Content-Length" in response.headers:
-            try:
-                total_size = int(response.headers["Content-Length"])
-            except ValueError:
-                total_size = 0
-
+        total_size = downloader.total_size
+        if total_size > 0:
             fetched_size = 0
             last_percent = 0
 
@@ -101,17 +121,44 @@ def download_file(
             # but skip the updating
             set_indicator = total_size > 1024 * 1024
 
-        with open(dest, "wb") as f:
-            for chunk in response.iter_content(chunk_size=chunk_size):
+        for fetched_size in downloader.download_with_progress(chunk_size):
+            if set_indicator:
+                percent = (fetched_size * 100) // total_size
+                if percent > last_percent:
+                    last_percent = percent
+                    update_context(f"Downloading {url} {percent:3}%")
+
+
+class Downloader:
+    def __init__(
+        self,
+        url: str,
+        dest: Path,
+        session: Authenticator | Session | None = None,
+    ):
+        self._dest = dest
+
+        get = requests.get if not session else session.get
+
+        self._response = get(url, stream=True, timeout=REQUESTS_TIMEOUT)
+        self._response.raise_for_status()
+
+    @cached_property
+    def total_size(self) -> int:
+        total_size = 0
+        if "Content-Length" in self._response.headers:
+            with suppress(ValueError):
+                total_size = int(self._response.headers["Content-Length"])
+        return total_size
+
+    def download_with_progress(self, chunk_size: int = 1024) -> Iterator[int]:
+        fetched_size = 0
+        with atomic_open(self._dest) as f:
+            for chunk in self._response.iter_content(chunk_size=chunk_size):
                 if chunk:
                     f.write(chunk)
-
-                    if set_indicator:
-                        fetched_size += len(chunk)
-                        percent = (fetched_size * 100) // total_size
-                        if percent > last_percent:
-                            last_percent = percent
-                            update_context(f"Downloading {url} {percent:3}%")
+                    fetched_size += len(chunk)
+                    yield fetched_size
 
 
 def get_package_version_display_string(
@@ -170,7 +217,8 @@ def _get_win_folder_from_registry(csidl_name: str) -> str:
     )
     dir, type = _winreg.QueryValueEx(key, shell_folder_name)
 
-    return cast(str, dir)
+    assert isinstance(dir, str)
+    return dir
 
 
 def _get_win_folder_with_ctypes(csidl_name: str) -> str:
@@ -218,7 +266,7 @@ def get_win_folder(csidl_name: str) -> Path:
     raise RuntimeError("Method can only be called on Windows.")
 
 
-def get_real_windows_path(path: str | Path) -> Path:
+def get_real_windows_path(path: Path) -> Path:
     program_files = get_win_folder("CSIDL_PROGRAM_FILES")
     local_appdata = get_win_folder("CSIDL_LOCAL_APPDATA")
 
@@ -233,3 +281,25 @@ def get_real_windows_path(path: str | Path) -> Path:
         path = path.resolve()
 
     return path
+
+
+def get_file_hash(path: Path, hash_name: str = "sha256") -> str:
+    h = hashlib.new(hash_name)
+    with path.open("rb") as fp:
+        for content in iter(lambda: fp.read(io.DEFAULT_BUFFER_SIZE), b""):
+            h.update(content)
+
+    return h.hexdigest()
+
+
+def extractall(source: Path, dest: Path, zip: bool) -> None:
+    """Extract all members from either a zip or tar archive."""
+    if zip:
+        with zipfile.ZipFile(source) as archive:
+            archive.extractall(dest)
+    else:
+        with tarfile.open(source) as archive:
+            if hasattr(tarfile, "data_filter"):
+                archive.extractall(dest, filter="data")
+            else:
+                archive.extractall(dest)
